/***************************************************************************
 *   Copyright (C) 2008 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "imagewidget.h"
#include <QPainter>
#include <QFileDialog>
#include <QResizeEvent>
#include <QMouseEvent>
#include <QRegion>
#include <QUrl>


#include <iostream>
ImageWidget::ImageWidget(QWidget *parent)
: QWidget(parent)
{
    m_baseSize = QSize(1200,527);
    m_rendered = false;
    m_pressed = false;

    setAcceptDrops(true);
}

ImageWidget::~ImageWidget()
{
}

void ImageWidget::paintEvent(QPaintEvent *e)
{
    
    if (m_backgroundCache.isNull())
    {
         QImage imageCache = QImage(size(), QImage::Format_ARGB32_Premultiplied);
         QPainter p(&imageCache);
         p.fillRect(rect(), Qt::gray);

         if (!m_scaledPixmap.isNull())
         {
             p.drawPixmap(m_imageRect.topLeft(), m_scaledPixmap);
        
             if (!m_rendered)
                  p.fillRect(rect(), QColor(40,40,40,200));
        }
        m_backgroundCache = QPixmap::fromImage(imageCache);
    }
    QPainter p(this);
	p.setClipRegion(e->region());
    p.drawPixmap(QPoint(0,0), m_backgroundCache);
    if (!m_rendered && !m_scaledPixmap.isNull())
    {
        QRect r = toWidget(m_selectionRect);
        p.save();
        p.setClipRect(r);
        p.drawPixmap(m_imageRect, m_scaledPixmap);
        p.restore();
        p.setPen(Qt::black);
        p.setBrush(Qt::NoBrush);
        p.drawRect(r);
    }
}

void ImageWidget::resizeEvent(QResizeEvent *e)
{
    Q_UNUSED(e);
    // reset the cache
    m_backgroundCache = QPixmap();

    m_scaledPixmap = QPixmap::fromImage(m_currentImage).scaled(size(), Qt::KeepAspectRatio);

    if (m_scaledPixmap.isNull())
        return;

    m_imageRect = m_scaledPixmap.rect();
    m_imageRect.moveCenter(rect().center());

    m_scale = (float) m_scaledPixmap.width() / (float) m_currentImage.width(); 

    update();
}

void ImageWidget::dragMoveEvent(QDragMoveEvent *e)
{
    if (e->mimeData()->hasImage() || e->mimeData()->hasUrls())
        e->acceptProposedAction();
}

void ImageWidget::dragEnterEvent(QDragEnterEvent *e)
{
    if (e->mimeData()->hasImage() || e->mimeData()->hasUrls())
        e->acceptProposedAction();
}

void ImageWidget::dropEvent(QDropEvent *e)
{
    const QMimeData *mimeData = e->mimeData();

    if (mimeData->hasImage()) 
        setImage(qvariant_cast<QImage>(mimeData->imageData()));
    else if (mimeData->hasUrls())
    {
        QUrl url = mimeData->urls().first();
        setImage(QImage(url.toLocalFile()));
    }

    e->acceptProposedAction();
}

void ImageWidget::mouseMoveEvent(QMouseEvent *e)
{
    if (m_pressed)
       moveSelection(e->pos());
}

void ImageWidget::mousePressEvent(QMouseEvent *e)
{
    m_pressed = true && !m_rendered;
    moveSelection(e->pos());
}

void ImageWidget::mouseReleaseEvent(QMouseEvent *e)
{
    m_pressed = false;
}

void ImageWidget::slotFileOpen()
{
     if (m_lastPath.isNull())
         m_lastPath = QDir::currentPath();
    QString fileName = QFileDialog::getOpenFileName(this, tr("Open Image"), m_lastPath);

    QImage image(fileName);

    if (image.isNull())
        return;
    
    m_lastPath = QDir(fileName).absolutePath();
    setImage(image);

    resizeEvent(&QResizeEvent(size(), size()));
}

void ImageWidget::slotFileSave()
{
     if (!m_rendered)
         return;
     if (m_lastPath.isNull())
         m_lastPath = QDir::currentPath();
     QString fileName = QFileDialog::getSaveFileName(this, tr("Save Image"), m_lastPath, tr("Images (*.png *.xpm *.jpg)"));
     if (!fileName.isNull())
     {
         m_currentImage.save(fileName);
         m_lastPath = QDir(fileName).absolutePath();
     }
         
}
   
void ImageWidget::slotApplyEffect()
{
    QImage selection = m_image.copy(m_selectionRect).scaled(m_baseSize, Qt::KeepAspectRatio).convertToFormat(QImage::Format_ARGB32_Premultiplied);

    QImage img(1200,800,QImage::Format_ARGB32_Premultiplied);
    
    QPainter p(&img);
    p.fillRect(img.rect(), Qt::black);
    p.drawImage(QPoint(0,74), selection);

    QImage reflect = selection.mirrored(false,true);

    QPainter p2(&reflect);
    QLinearGradient g(QPointF(0,0), QPointF(0,reflect.height()));
    g.setColorAt(0, QColor(0,0,0,128));
    g.setColorAt(0.3, QColor(Qt::transparent));
    g.setColorAt(1, QColor(Qt::transparent));
    
    p2.setCompositionMode(QPainter::CompositionMode_DestinationIn);
    p2.fillRect(reflect.rect(), g);

    p.drawImage(QPoint(0, 74 + m_baseSize.height() + 5), reflect);
   
    m_currentImage = img;
    m_rendered = true;
    emit enableDisable();
    resizeEvent(&QResizeEvent(size(), size()));
}

void ImageWidget::slotReset()
{
    setImage(m_image);
}

void ImageWidget::slotCurrentChanged(int value)
{
    float scale = (float)m_baseSize.height() / (float)m_baseSize.width();
    m_selectionSize.setWidth(value);
    m_selectionSize.setHeight(value * scale);
    QPoint center = m_selectionRect.center();
    m_selectionRect.setSize(m_selectionSize);
    m_selectionRect.moveCenter(center);
    checkSelectionBounds();
    update();
}

bool ImageWidget::isRendered()
{
    return m_rendered;
}

void ImageWidget::setImage(const QImage &img)
{
    m_image = m_currentImage = img;
    m_rendered = false;

    m_selectionSize = m_baseSize;
    m_selectionSize.scale(img.size(), Qt::KeepAspectRatio);
    m_selectionRect.setSize(m_selectionSize);
    m_selectionRect.moveCenter(m_image.rect().center());
    emit currentChanged(m_selectionSize.width());
    emit enableDisable();
    emit maxChanged(m_image.width());
    emit currentChanged(m_image.width());
    resizeEvent(&QResizeEvent(size(),size()));
}

void ImageWidget::moveSelection(const QPoint &center)
{
    m_selectionRect.moveCenter(fromWidget(center));
    checkSelectionBounds();
    update();
}

void ImageWidget::checkSelectionBounds()
{
    QRect bounds = m_currentImage.rect();
    QRect r = m_selectionRect;
    if (r.top() < bounds.top())
        r.moveTop(bounds.top());
    if (r.left() < bounds.left())
        r.moveLeft(bounds.left());
    if (r.bottom() > bounds.bottom())
        r.moveBottom(bounds.bottom());
    if (r.right() > bounds.right())
        r.moveRight(bounds.right());
    m_selectionRect = r;
}

QPoint ImageWidget::toWidget(const QPoint &p)
{
    QPoint result;
    result.setX(p.x() * m_scale + m_imageRect.left());
    result.setY(p.y() * m_scale + m_imageRect.top());
    return result;
}

QPoint ImageWidget::fromWidget(const QPoint &p)
{
    QPoint result;
    result.setX((p.x() - m_imageRect.left()) / m_scale);
    result.setY((p.y() - m_imageRect.top()) / m_scale);

    return result;
}

QRect  ImageWidget::toWidget(const QRect &r)
{
    QRect result;
    QPoint center = toWidget(r.center());

    result.setWidth(r.width() * m_scale);
    result.setHeight(r.height() * m_scale);
    result.moveCenter(center);

    return result;
}
